/**
 * Add RAI support to MUCs.
 * @see https://xmpp.org/extensions/inbox/room-activity-indicators.html
 */
converse.plugins.add('converse-rai', {
  dependencies: [
    'converse-chatboxes',
  ],
  overrides: {
    ChatRoom: {
      async initialize() {
        await this.__super__.initialize.apply(this, arguments);
        this.listenTo(this, 'change:hidden', this.onHiddenChange);
      },

      /**
       * Subscribe to RAI if connected to the room, but it's hidden.
       * @private
       * @method _converse.ChatRoom#onConnectionStatusChanged
       */
      async onConnectionStatusChanged() {
        console.log('ChatRoom#onConnectionStatusChanged');
        const { api } = this.__super__._converse;
        if (this.session.get('connection_status') === converse.ROOMSTATUS.ENTERED) {
          if (this.get('hidden') && api.settings.get('muc_subscribe_to_rai') && this.getOwnAffiliation() !== 'none') {
            await this.leave();
            this.enableRAI();
          } else {
            await this.__super__.onConnectionStatusChanged.apply(this, arguments);
          }
        }
      },

      /**
       * Handler that gets called when the 'hidden' flag is toggled.
       * @private
       * @method _converse.ChatRoom#onHiddenChange
       */
      async onHiddenChange() {
        console.log('ChatRoomView#onHiddenChange');
        const conn_status = this.session.get('connection_status');
        const { api } = this.__super__._converse;

        if (this.get('hidden') && conn_status === converse.ROOMSTATUS.ENTERED) {
          if (api.settings.get('muc_subscribe_to_rai') && this.getOwnAffiliation() !== 'none') {
            if (conn_status !== converse.ROOMSTATUS.DISCONNECTED) {
              this.sendMarkerForLastMessage('received', true);
              await this.leave();
            }
            this.enableRAI();
          }
        } else if (conn_status === converse.ROOMSTATUS.DISCONNECTED) {
          this.rejoin();
        }
      },

      /**
       * Ensures that the user is subscribed to XEP-0437 Room Activity Indicators
       * if `muc_subscribe_to_rai` is set to `true`.
       * Only affiliated users can subscribe to RAI, but this method doesn't
       * check whether the current user is affiliated because it's intended to be
       * called after the MUC has been left and we don't have that information
       * anymore.
       * @private
       * @method _converse.ChatRoom#enableRAI
       */
      enableRAI() {
        console.log('ChatRoom#enableRAI');
        const { api } = this.__super__._converse;
        if (api.settings.get('muc_subscribe_to_rai')) {
          api.rooms.subscribe(this.get('jid'));
        }
      },
    },
    ChatBox: {
      /**
       * Finds the last eligible message and then sends a XEP-0333 chat marker for it.
       * @param { ('received'|'displayed'|'acknowledged') } [type='displayed']
       * @param { Boolean } force - Whether a marker should be sent for the
       *  message, even if it didn't include a `markable` element.
       */
      sendMarkerForLastMessage(type = 'displayed', force = false) {
        const msgs = Array.from(this.messages.models);
        msgs.reverse();
        const msg = msgs.find(m => m.get('sender') === 'them' && (force || m.get('is_markable')));
        msg && this.sendMarkerForMessage(msg, type, force);
      },
    },
  },
  initialize() {
    const { Strophe, _, $pres, u } = converse.env;
    const _converse = this._converse;
    const { api } = _converse;

    // Register namespace
    Strophe.addNamespace('RAI', 'urn:xmpp:rai:0');

    api.listen.on('addClientFeatures', () => {
      _converse.api.disco.own.features.add(Strophe.NS.RAI);
    });

    // Register settings
    api.settings.extend({
      muc_subscribe_to_rai: true,
    });

    Object.assign(api.rooms, {
      /**
       * Send an RAI stanza for the given jids.
       * The presence stanza is sent for a whole muc domain.
       *
       * @param {string|Array} jids
       * @returns {void}
       */
      subscribe(jids) {
        if (typeof jids === 'string') {
          jids = [jids];
        }

        const muc_domains = jids.map(jid => Strophe.getDomainFromJid(jid));

        _.uniq(muc_domains).forEach(muc_domain => {
          const rai = $pres({ to: muc_domain, id: u.getUniqueId() }).c('rai', {
            'xmlns': Strophe.NS.RAI,
          });
          api.send(rai);
          console.log('Sent RAI stanza for muc_domain', muc_domain, rai.toString());
        });
      },
    });

    function mucActivityHandler(message) {
      console.log('mucActivityHandler', message);
      const from_jid = message.attributes.from?.nodevalue;
      const room_jid = from_jid?.split('/')[0];
      let room = '';
      let ignore = false;

      for (let i = 0; i < _converse.chatboxes.models.length; i++) {
        if (_converse.chatboxes.models[i].id === room_jid) {
          room = _converse.chatboxes.models[i].id;
          break;
        }
      }

      if (room && from_jid && room_jid) {
        if (from_jid === room_jid + '/' + room.get('nick')) {
          ignore = true;
        }
      }

      if (message && !ignore) {
        message.querySelectorAll('activity').forEach(activity => {
          if (activity && activity.namespaceURI === Strophe.NS.RAI) {
            const jid = activity.textContent;
            // TODO: Give the chatroom object instead of the jid
            api.trigger('chatRoomHasActivity', jid);
          }
        });
      }

      return true;
    }

    // TODO: On loading subscribe to all the ChatViews in navigation (into another plugin)
    api.listen.on('connected', () => {
      _converse.connection.addHandler(mucActivityHandler, null, 'message', 'groupchat');
    });

    // TODO: Remove
    // api.listen.on('send', stanza => console.log('send', stanza));
  },
});
